----------------------------------------------------------------------------------
-- Company: University of Connecticut
-- Engineer: Brendan Krueger
-- 
-- Create Date:    16:50:41 06/18/2007 
-- Design Name: DAC (AD5535) serial communication block
-- Module Name:    DAC_box - DAC_box_arch 
-- Project Name: DAC Box
-- Target Devices: Xilinx Spartan-3A
-- Tool versions: Xilinx ISE WebPACK 9.1.03i
-- Description: A composite block of the DAC controller circuit and the DAC
--						emulator circuit for testing of the DAC controller
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity DAC_box is
    Port ( CLK				:	in	STD_LOGIC;
			  -- clock
           invRst			:	in	STD_LOGIC;
			  -- asynchronous, active-low reset line for controller
           invDRst_in	:	in	STD_LOGIC;
			  -- asynchronous, active-low reset line for DAC
           invTxGo		:	in	STD_LOGIC;
			  -- active-low pulse to signal start of transmission
           Addr			:	in	STD_LOGIC_VECTOR (4 downto 0);
			  -- 5-bit address bus
           Code			:	in	STD_LOGIC_VECTOR (13 downto 0);
			  -- 14-bit code bus
           ch00 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch01 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch02 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch03 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch04 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch05 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch06 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch07 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch08 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch09 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch10 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch11 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch12 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch13 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch14 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch15 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch16 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch17 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch18 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch19 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch20 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch21 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch22 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch23 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch24 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch25 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch26 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch27 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch28 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch29 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch30 : out  STD_LOGIC_VECTOR (13 downto 0);
           ch31 : out  STD_LOGIC_VECTOR (13 downto 0));
			  -- 32 14-bit DAC output channels
end DAC_box;

architecture DAC_box_arch of DAC_box is

	component DAC_controller
	-- declare a DAC controller wrapper
		Port	(	CLK			: in  STD_LOGIC;
					invRst		: in  STD_LOGIC;
					invDRst_in	: in  STD_LOGIC;
					invTxGo		: in  STD_LOGIC;
					Addr			: in  STD_LOGIC_VECTOR (4 downto 0);
					Code			: in  STD_LOGIC_VECTOR (13 downto 0);
					SCLK			: out  STD_LOGIC;
					invSYNC		: out  STD_LOGIC;
					Data 			: out  STD_LOGIC;
					invDRst_out : out  STD_LOGIC);
	end component;
	
	component DAC_emulator
	-- declare a DAC emulator wrapper
		Port (	SCLK		:	in		STD_LOGIC;
					invRESET	:	in		STD_LOGIC;
					invSYNC	:	in		STD_LOGIC;
					D_in		:	in		STD_LOGIC;
					ch00 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch01 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch02 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch03 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch04 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch05 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch06 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch07 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch08 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch09 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch10 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch11 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch12 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch13 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch14 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch15 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch16 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch17 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch18 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch19 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch20 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch21 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch22 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch23 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch24 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch25 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch26 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch27 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch28 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch29 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch30 : out  STD_LOGIC_VECTOR (13 downto 0);
					ch31 : out  STD_LOGIC_VECTOR (13 downto 0));
	end component;

	signal SerialReset : STD_LOGIC;
	-- /RESET
	signal SerialClock : STD_LOGIC;
	-- SCLK
	signal SerialSynch : STD_LOGIC;
	-- /SYNC
	signal SerialData  : STD_LOGIC;
	-- D_in
	
begin

	-- port maps to include components
	u1: DAC_controller	port map (CLK, invRst, invDRst_in, invTxGo, Addr, Code,
											SerialClock, SerialSynch, SerialData, SerialReset);
	u2: DAC_emulator		port map (SerialClock, SerialReset, SerialSynch, SerialData, ch00, ch01,
											ch02, ch03, ch04, ch05, ch06, ch07,
											ch08, ch09, ch10, ch11, ch12, ch13,
											ch14, ch15, ch16, ch17, ch18, ch19,
											ch20, ch21, ch22, ch23, ch24, ch25,
											ch26, ch27, ch28, ch29, ch30, ch31);

end DAC_box_arch;

