----------------------------------------------------------------------------------
-- Company: University of Connecticut
-- Engineer: Brendan Krueger
-- 
-- Create Date:    14:16:57 06/20/2007 
-- Design Name: Single Cycle Delayer for DAC Controller
-- Module Name:    DAC_delay - DAC_delay_arch 
-- Project Name: Readout Electronics Digital Board
-- Target Devices: Xilinx Spartan-3A
-- Tool versions: Xilinx ISE WebPACK 9.1.03i
-- Description: Delays a signal by one clock cycle
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity DAC_delay is
    Port ( CLK : in  STD_LOGIC;			-- clock
           SigIn : in  STD_LOGIC;		-- input signal
           SigOut : out  STD_LOGIC);	-- output signal
end DAC_delay;

architecture DAC_delay_arch of DAC_delay is

	signal temp1 : STD_LOGIC;		-- first flip-flop
	signal temp2 : STD_LOGIC;		-- second flip-flop

begin

	delayer : process (CLK, SigIn, temp1, temp2)
	begin
		if (rising_edge(CLK)) then
			temp2 <= temp1;		-- connect first flip-flop to second flip-flop
			temp1 <= SigIn;		-- connect input to first flip flop
		else
			temp2 <= temp2;
			temp1 <= temp1;
		end if;
	end process delayer;
	
	SigOut <= temp2;		-- connect second flip-flop to output

end DAC_delay_arch;

